@@ -448,6 +448,7 @@ QList<KPluginInfo> KPluginInfo::fromFiles(const QStringList &files, const KConfi
     return infolist;
 }
 
+#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 81)
 QList<KPluginInfo> KPluginInfo::fromKPartsInstanceName(const QString &name, const KConfigGroup &config)
 {
     QStringList files;
@@ -461,6 +462,7 @@ QList<KPluginInfo> KPluginInfo::fromKPartsInstanceName(const QString &name, cons
     }
     return fromFiles(files, config);
 }
+#endif
 
 bool KPluginInfo::isHidden() const
 {
@@ -644,8 +646,9 @@ QVariant KPluginInfo::property(const QString &key) const
                     newResult += value.toString();
                 }
                 result = newResult;
-            } else
+            } else {
                 qCWarning(SERVICES) << "Cannot interpret" << result << "into a string list";
+            }
         }
         return result;
     }
