@@ -7,8 +7,8 @@
 */
 
 #include "kservice.h"
-#include "kservice_p.h"
 #include "kmimetypefactory_p.h"
+#include "kservice_p.h"
 #include "ksycoca.h"
 #include "ksycoca_p.h"
 
@@ -19,13 +19,13 @@
 #include <QMimeDatabase>
 
 #include <KAuthorized>
-#include <KDesktopFile>
 #include <KConfigGroup>
+#include <KDesktopFile>
 #include <KLocalizedString>
 #include <KShell>
 
-#include <QStandardPaths>
 #include <QDebug>
+#include <QStandardPaths>
 
 #include "kservicefactory_p.h"
 #include "kservicetypefactory_p.h"
@@ -53,7 +53,7 @@ void KServicePrivate::init(const KDesktopFile *config, KService *q)
     QMap<QString, QString> entryMap = desktopGroup.entryMap();
 
     entryMap.remove(QStringLiteral("Encoding")); // reserved as part of Desktop Entry Standard
-    entryMap.remove(QStringLiteral("Version"));  // reserved as part of Desktop Entry Standard
+    entryMap.remove(QStringLiteral("Version")); // reserved as part of Desktop Entry Standard
 
     q->setDeleted(desktopGroup.readEntry("Hidden", false));
     entryMap.remove(QStringLiteral("Hidden"));
@@ -85,9 +85,7 @@ void KServicePrivate::init(const KDesktopFile *config, KService *q)
           return;*/
         m_strType = QStringLiteral("Application");
     } else if (m_strType != QLatin1String("Application") && m_strType != QLatin1String("Service")) {
-        qCWarning(SERVICES) << "The desktop entry file" << entryPath
-                   << "has Type=" << m_strType
-                   << "instead of \"Application\" or \"Service\"";
+        qCWarning(SERVICES) << "The desktop entry file" << entryPath << "has Type=" << m_strType << "instead of \"Application\" or \"Service\"";
         m_bValid = false;
         return;
     }
@@ -117,22 +115,16 @@ void KServicePrivate::init(const KDesktopFile *config, KService *q)
 
     const QStandardPaths::StandardLocation resource = config->resource();
 
-    if ((m_strType == QLatin1String("Application")) &&
-            (resource != QStandardPaths::ApplicationsLocation) &&
-            !absPath) {
-        qCWarning(SERVICES) << "The desktop entry file" << entryPath
-                   << "has Type=" << m_strType << "but is located under \"" << QStandardPaths::displayName(resource)
-                   << "\" instead of \"Applications\"";
+    if ((m_strType == QLatin1String("Application")) && (resource != QStandardPaths::ApplicationsLocation) && !absPath) {
+        qCWarning(SERVICES) << "The desktop entry file" << entryPath << "has Type=" << m_strType << "but is located under \""
+                            << QStandardPaths::displayName(resource) << "\" instead of \"Applications\"";
         m_bValid = false;
         return;
     }
 
-    if ((m_strType == QLatin1String("Service")) &&
-            (resource != QStandardPaths::GenericDataLocation) &&
-            !absPath) {
-        qCWarning(SERVICES) << "The desktop entry file" << entryPath
-                   << "has Type=" << m_strType << "but is located under \"" << QStandardPaths::displayName(resource)
-                   << "\" instead of \"Shared Data\"/kservices5";
+    if ((m_strType == QLatin1String("Service")) && (resource != QStandardPaths::GenericDataLocation) && !absPath) {
+        qCWarning(SERVICES) << "The desktop entry file" << entryPath << "has Type=" << m_strType << "but is located under \""
+                            << QStandardPaths::displayName(resource) << "\" instead of \"Shared Data\"/kservices5";
         m_bValid = false;
         return;
     }
@@ -143,9 +135,9 @@ void KServicePrivate::init(const KDesktopFile *config, KService *q)
 
     m_strIcon = config->readIcon();
     entryMap.remove(QStringLiteral("Icon"));
-    m_bTerminal = desktopGroup.readEntry("Terminal", false);  // should be a property IMHO
+    m_bTerminal = desktopGroup.readEntry("Terminal", false); // should be a property IMHO
     entryMap.remove(QStringLiteral("Terminal"));
-    m_strTerminalOptions = desktopGroup.readEntry("TerminalOptions");   // should be a property IMHO
+    m_strTerminalOptions = desktopGroup.readEntry("TerminalOptions"); // should be a property IMHO
     entryMap.remove(QStringLiteral("TerminalOptions"));
     m_strWorkingDirectory = KShell::tildeExpand(config->readPath());
     entryMap.remove(QStringLiteral("Path"));
@@ -174,9 +166,9 @@ void KServicePrivate::init(const KDesktopFile *config, KService *q)
     entryMap.remove(QStringLiteral("X-KDE-Library"));
     if (!m_strLibrary.isEmpty() && m_strType == QLatin1String("Application")) {
         qCWarning(SERVICES) << "The desktop entry file" << entryPath << "has Type=" << m_strType
-                   << "but also has a X-KDE-Library key. This works for now,"
-                   " but makes user-preference handling difficult, so support for this might"
-                   " be removed at some point. Consider splitting it into two desktop files.";
+                            << "but also has a X-KDE-Library key. This works for now,"
+                               " but makes user-preference handling difficult, so support for this might"
+                               " be removed at some point. Consider splitting it into two desktop files.";
     }
 
     QStringList lstServiceTypes = desktopGroup.readEntry("ServiceTypes", QStringList());
@@ -187,7 +179,7 @@ void KServicePrivate::init(const KDesktopFile *config, KService *q)
     entryMap.remove(QStringLiteral("MimeType"));
 
     if (m_strType == QLatin1String("Application") && !lstServiceTypes.contains(QLatin1String("Application")))
-        // Applications implement the service type "Application" ;-)
+    // Applications implement the service type "Application" ;-)
     {
         lstServiceTypes += QStringLiteral("Application");
     }
@@ -235,11 +227,12 @@ void KServicePrivate::init(const KDesktopFile *config, KService *q)
     m_strDesktopEntryName = _name.toLower();
 
     // Exec lines from the KCMs always have the pattern "<program> m_strDesktopEntryName", see https://phabricator.kde.org/T13729
+    const static bool hasSystemsettings = !QStandardPaths::findExecutable(QStringLiteral("systemsettings5")).isEmpty();
     if (m_strExec.isEmpty() && serviceTypes().contains(QStringLiteral("KCModule"))) {
         if (desktopGroup.readEntry("X-KDE-ParentApp") == QLatin1String("kinfocenter")) {
             m_strExec = QStringLiteral("kinfocenter ") + m_strDesktopEntryName;
         } else if (desktopGroup.readEntry("X-KDE-ParentApp") == QLatin1String("kcontrol")) {
-            if (!desktopGroup.readEntry("X-KDE-System-Settings-Parent-Category").isEmpty()) {
+            if (!desktopGroup.readEntry("X-KDE-System-Settings-Parent-Category").isEmpty() && hasSystemsettings) {
                 m_strExec = QStringLiteral("systemsettings5 ") + m_strDesktopEntryName;
             } else {
                 m_strExec = QStringLiteral("kcmshell5 ") + m_strDesktopEntryName;
@@ -261,13 +254,13 @@ void KServicePrivate::init(const KDesktopFile *config, KService *q)
     // Store all additional entries in the property map.
     // A QMap<QString,QString> would be easier for this but we can't
     // break BC, so we have to store it in m_mapProps.
-//  qDebug("Path = %s", entryPath.toLatin1().constData());
+    //  qDebug("Path = %s", entryPath.toLatin1().constData());
     QMap<QString, QString>::ConstIterator it = entryMap.constBegin();
     for (; it != entryMap.constEnd(); ++it) {
         const QString key = it.key();
         // do not store other translations like Name[fr]; kbuildsycoca will rerun if we change languages anyway
         if (!key.contains(QLatin1Char('['))) {
-            //qCDebug(SERVICES) << "  Key =" << key << " Data =" << *it;
+            // qCDebug(SERVICES) << "  Key =" << key << " Data =" << *it;
             if (key == QLatin1String("X-Flatpak-RenamedFrom")) {
                 m_mapProps.insert(key, desktopGroup.readXdgListEntry(key));
             } else {
@@ -300,11 +293,8 @@ void KServicePrivate::parseActions(const KDesktopFile *config, KService *q)
             if (!cg.hasKey("Name") || !cg.hasKey("Exec")) {
                 qCWarning(SERVICES) << "The action" << group << "in the desktop file" << q->entryPath() << "has no Name or no Exec key";
             } else {
-                m_actions.append(KServiceAction(group,
-                                                cg.readEntry("Name"),
-                                                cg.readEntry("Icon"),
-                                                cg.readEntry("Exec"),
-                                                cg.readEntry("NoDisplay", false), serviceClone));
+                m_actions.append(
+                    KServiceAction(group, cg.readEntry("Name"), cg.readEntry("Icon"), cg.readEntry("Exec"), cg.readEntry("NoDisplay", false), serviceClone));
             }
         } else {
             qCWarning(SERVICES) << "The desktop file" << q->entryPath() << "references the action" << group << "but doesn't define it";
@@ -314,13 +304,16 @@ void KServicePrivate::parseActions(const KDesktopFile *config, KService *q)
 
 void KServicePrivate::load(QDataStream &s)
 {
-    qint8 def, term;
-    qint8 dst, initpref;
+    qint8 def;
+    qint8 term;
+    qint8 dst;
+    qint8 initpref;
 
     // WARNING: THIS NEEDS TO REMAIN COMPATIBLE WITH PREVIOUS KService 5.x VERSIONS!
     // !! This data structure should remain binary compatible at all times !!
     // You may add new fields at the end. Make sure to update KSYCOCA_VERSION
     // number in ksycoca.cpp
+    // clang-format off
     s >> m_strType >> m_strName >> m_strExec >> m_strIcon
       >> term >> m_strTerminalOptions
       >> m_strWorkingDirectory >> m_strComment >> def >> m_mapProps
@@ -331,6 +324,7 @@ void KServicePrivate::load(QDataStream &s)
       >> m_lstKeywords >> m_strGenName
       >> categories >> menuId >> m_actions >> m_serviceTypes
       >> m_lstFormFactors;
+    // clang-format on
 
     m_bAllowAsDefault = bool(def);
     m_bTerminal = bool(term);
@@ -343,7 +337,8 @@ void KServicePrivate::load(QDataStream &s)
 void KServicePrivate::save(QDataStream &s)
 {
     KSycocaEntryPrivate::save(s);
-    qint8 def = m_bAllowAsDefault, initpref = m_initialPreference;
+    qint8 def = m_bAllowAsDefault;
+    qint8 initpref = m_initialPreference;
     qint8 term = m_bTerminal;
     qint8 dst = qint8(m_DBUSStartusType);
 
@@ -351,15 +346,8 @@ void KServicePrivate::save(QDataStream &s)
     // !! This data structure should remain binary compatible at all times !!
     // You may add new fields at the end. Make sure to update KSYCOCA_VERSION
     // number in ksycoca.cpp
-    s << m_strType << m_strName << m_strExec << m_strIcon
-      << term << m_strTerminalOptions
-      << m_strWorkingDirectory << m_strComment << def << m_mapProps
-      << m_strLibrary
-      << dst
-      << m_strDesktopEntryName
-      << initpref
-      << m_lstKeywords << m_strGenName
-      << categories << menuId << m_actions << m_serviceTypes
+    s << m_strType << m_strName << m_strExec << m_strIcon << term << m_strTerminalOptions << m_strWorkingDirectory << m_strComment << def << m_mapProps
+      << m_strLibrary << dst << m_strDesktopEntryName << initpref << m_lstKeywords << m_strGenName << categories << menuId << m_actions << m_serviceTypes
       << m_lstFormFactors;
 }
 
@@ -406,7 +394,7 @@ KService::KService(QDataStream &_str, int _offset)
 }
 
 KService::KService(const KService &other)
-   : KSycocaEntry(*new KServicePrivate(*static_cast<KServicePrivate *>(other.d_ptr)))
+    : KSycocaEntry(*new KServicePrivate(*other.d_func()))
 {
 }
 
@@ -419,7 +407,7 @@ bool KService::hasServiceType(const QString &serviceType) const
     Q_D(const KService);
 
     if (!d->m_bValid) {
-        return false;    // (useless) safety test
+        return false; // (useless) safety test
     }
     const KServiceType::Ptr ptr = KServiceType::serviceType(serviceType);
     if (!ptr) {
@@ -427,7 +415,7 @@ bool KService::hasServiceType(const QString &serviceType) const
     }
     const int serviceOffset = offset();
     // doesn't seem to work:
-    //if ( serviceOffset == 0 )
+    // if ( serviceOffset == 0 )
     //    serviceOffset = serviceByStorageId( storageId() );
     if (serviceOffset) {
         KSycoca::self()->ensureCacheValid();
@@ -440,7 +428,7 @@ bool KService::hasServiceType(const QString &serviceType) const
     QVector<ServiceTypeAndPreference>::ConstIterator it = d->m_serviceTypes.begin();
     for (; it != d->m_serviceTypes.end(); ++it) {
         const QString &st = (*it).serviceType;
-        //qCDebug(SERVICES) << "    has " << (*it);
+        // qCDebug(SERVICES) << "    has " << (*it);
         if (st == ptr->name()) {
             return true;
         }
@@ -477,7 +465,7 @@ bool KService::hasMimeType(const QString &mimeType) const
     QVector<ServiceTypeAndPreference>::ConstIterator it = d->m_serviceTypes.begin();
     for (; it != d->m_serviceTypes.end(); ++it) {
         const QString &st = (*it).serviceType;
-        //qCDebug(SERVICES) << "    has " << (*it);
+        // qCDebug(SERVICES) << "    has " << (*it);
         if (st == mime) {
             return true;
         }
@@ -513,9 +501,9 @@ QVariant KService::property(const QString &_name, QVariant::Type t) const
 QVariant KServicePrivate::property(const QString &_name, QVariant::Type t) const
 {
     if (_name == QLatin1String("Type")) {
-        return QVariant(m_strType);    // can't be null
+        return QVariant(m_strType); // can't be null
     } else if (_name == QLatin1String("Name")) {
-        return QVariant(m_strName);    // can't be null
+        return QVariant(m_strName); // can't be null
     } else if (_name == QLatin1String("Exec")) {
         return makeStringVariant(m_strExec);
     } else if (_name == QLatin1String("Icon")) {
@@ -541,7 +529,7 @@ QVariant KServicePrivate::property(const QString &_name, QVariant::Type t) const
     } else if (_name == QLatin1String("DesktopEntryPath")) { // can't be null
         return QVariant(path);
     } else if (_name == QLatin1String("DesktopEntryName")) {
-        return QVariant(m_strDesktopEntryName);    // can't be null
+        return QVariant(m_strDesktopEntryName); // can't be null
     } else if (_name == QLatin1String("Categories")) {
         return QVariant(categories);
     } else if (_name == QLatin1String("Keywords")) {
@@ -566,7 +554,7 @@ QVariant KServicePrivate::property(const QString &_name, QVariant::Type t) const
 
     QMap<QString, QVariant>::ConstIterator it = m_mapProps.find(_name);
     if ((it == m_mapProps.end()) || (!it->isValid())) {
-        //qCDebug(SERVICES) << "Property not found " << _name;
+        // qCDebug(SERVICES) << "Property not found " << _name;
         return QVariant(); // No property set.
     }
 
@@ -667,16 +655,22 @@ bool KService::showInCurrentDesktop() const
     Q_D(const KService);
 
     const QString envVar = QString::fromLatin1(qgetenv("XDG_CURRENT_DESKTOP"));
-#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
-    QVector<QStringRef> currentDesktops(envVar.splitRef(QLatin1Char(':'), QString::SkipEmptyParts));
+
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+    QVector<QStringView> currentDesktops = QStringView(envVar).split(QLatin1Char(':'), Qt::SkipEmptyParts);
 #else
-    QVector<QStringRef> currentDesktops(envVar.splitRef(QLatin1Char(':'), Qt::SkipEmptyParts));
+    QVector<QStringRef> currentDesktops = envVar.splitRef(QLatin1Char(':'), Qt::SkipEmptyParts);
 #endif
+
     const QString kde = QStringLiteral("KDE");
     if (currentDesktops.isEmpty()) {
         // This could be an old display manager, or e.g. a failsafe session with no desktop name
         // In doubt, let's say we show KDE stuff.
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+        currentDesktops.append(kde);
+#else
         currentDesktops.append(&kde);
+#endif
     }
 
     // This algorithm is described in the desktop entry spec
@@ -684,7 +678,7 @@ bool KService::showInCurrentDesktop() const
     QMap<QString, QVariant>::ConstIterator it = d->m_mapProps.find(QStringLiteral("OnlyShowIn"));
     if ((it != d->m_mapProps.end()) && (it->isValid())) {
         const QStringList aList = it->toString().split(QLatin1Char(';'));
-        for (const QStringRef &desktop : qAsConst(currentDesktops)) {
+        for (const auto &desktop : std::as_const(currentDesktops)) {
             if (aList.contains(desktop)) {
                 return true;
             }
@@ -695,7 +689,7 @@ bool KService::showInCurrentDesktop() const
     it = d->m_mapProps.find(QStringLiteral("NotShowIn"));
     if ((it != d->m_mapProps.end()) && (it->isValid())) {
         const QStringList aList = it->toString().split(QLatin1Char(';'));
-        for (const QStringRef &desktop : qAsConst(currentDesktops)) {
+        for (const auto &desktop : std::as_const(currentDesktops)) {
             if (aList.contains(desktop)) {
                 return false;
             }
@@ -757,6 +751,7 @@ QString KService::untranslatedGenericName() const
     return v.isValid() ? v.toString() : QString();
 }
 
+#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 87)
 QString KService::parentApp() const
 {
     Q_D(const KService);
@@ -767,7 +762,9 @@ QString KService::parentApp() const
 
     return it->toString();
 }
+#endif
 
+#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 87)
 QString KService::pluginKeyword() const
 {
     Q_D(const KService);
@@ -778,6 +775,7 @@ QString KService::pluginKeyword() const
 
     return it->toString();
 }
+#endif
 
 QString KService::docPath() const
 {
@@ -797,8 +795,10 @@ bool KService::allowMultipleFiles() const
 {
     Q_D(const KService);
     // Can we pass multiple files on the command line or do we have to start the application for every single file ?
-    return (d->m_strExec.contains(QLatin1String("%F")) || d->m_strExec.contains(QLatin1String("%U")) ||
-            d->m_strExec.contains(QLatin1String("%N")) || d->m_strExec.contains(QLatin1String("%D")));
+    return (d->m_strExec.contains(QLatin1String("%F")) //
+            || d->m_strExec.contains(QLatin1String("%U")) //
+            || d->m_strExec.contains(QLatin1String("%N")) //
+            || d->m_strExec.contains(QLatin1String("%D")));
 }
 
 QStringList KService::categories() const
@@ -829,16 +829,16 @@ QString KService::storageId() const
 QString KService::locateLocal() const
 {
     Q_D(const KService);
-    if (d->menuId.isEmpty() || entryPath().startsWith(QLatin1String(".hidden")) ||
-            (QDir::isRelativePath(entryPath()) && d->categories.isEmpty())) {
+    if (d->menuId.isEmpty() //
+        || entryPath().startsWith(QLatin1String(".hidden")) //
+        || (QDir::isRelativePath(entryPath()) && d->categories.isEmpty())) {
         return KDesktopFile::locateLocal(entryPath());
     }
 
     return QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QLatin1String("/applications/") + d->menuId;
 }
 
-QString KService::newServicePath(bool showInMenu, const QString &suggestedName,
-                                 QString *menuId, const QStringList *reservedMenuIds)
+QString KService::newServicePath(bool showInMenu, const QString &suggestedName, QString *menuId, const QStringList *reservedMenuIds)
 {
     Q_UNUSED(showInMenu); // TODO KDE5: remove argument
 
@@ -884,8 +884,7 @@ QString KService::exec() const
 {
     Q_D(const KService);
     if (d->m_strType == QLatin1String("Application") && d->m_strExec.isEmpty()) {
-        qCWarning(SERVICES) << "The desktop entry file" << entryPath()
-                   << "has Type=" << d->m_strType << "but has no Exec field.";
+        qCWarning(SERVICES) << "The desktop entry file" << entryPath() << "has Type=" << d->m_strType << "but has no Exec field.";
     }
     return d->m_strExec;
 }
@@ -916,8 +915,13 @@ bool KService::terminal() const
 
 bool KService::runOnDiscreteGpu() const
 {
-    QVariant v = property(QStringLiteral("X-KDE-RunOnDiscreteGpu"), QVariant::Bool);
-    return v.isValid() && v.toBool();
+    QVariant prop = property(QStringLiteral("PrefersNonDefaultGPU"), QVariant::Bool);
+    if (!prop.isValid()) {
+        // For backwards compatibility
+        prop = property(QStringLiteral("X-KDE-RunOnDiscreteGpu"), QVariant::Bool);
+    }
+
+    return prop.isValid() && prop.toBool();
 }
 
 QString KService::desktopEntryName() const
@@ -1032,6 +1036,16 @@ void KService::setExec(const QString &exec)
     }
 }
 
+void KService::setWorkingDirectory(const QString &workingDir)
+{
+    Q_D(KService);
+
+    if (!workingDir.isEmpty()) {
+        d->m_strWorkingDirectory = workingDir;
+        d->path.clear();
+    }
+}
+
 QVector<KService::ServiceTypeAndPreference> &KService::_k_accessServiceTypes()
 {
     Q_D(KService);
@@ -1044,6 +1058,8 @@ QList<KServiceAction> KService::actions() const
     return d->m_actions;
 }
 
+#if KCOREADDONS_BUILD_DEPRECATED_SINCE(5, 86)
+#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 86)
 KService::operator KPluginName() const
 {
     if (!isValid()) {
@@ -1056,3 +1072,5 @@ KService::operator KPluginName() const
 
     return KPluginName(library());
 }
+#endif
+#endif
