@@ -1,21 +1,9 @@
-/* This file is part of the KDE project
-   Copyright (C) 1998, 1999 Torben Weis <weis@kde.org>
-                       1999 Waldo Bastian <bastian@kde.org>
+/*
+    This file is part of the KDE project
+    SPDX-FileCopyrightText: 1998, 1999 Torben Weis <weis@kde.org>
+    SPDX-FileCopyrightText: 1999 Waldo Bastian <bastian@kde.org>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #ifndef __kservicetype_h__
@@ -23,9 +11,9 @@
 
 #include <ksycocaentry.h>
 
+#include <QMap>
 #include <QString>
 #include <QStringList>
-#include <QMap>
 #include <QVariant>
 
 #include <KConfig>
@@ -154,14 +142,14 @@ private:
     friend class KServiceTypeFactory;
     /**
      * @internal construct a service from a stream.
-     * The stream must already be positionned at the correct offset
+     * The stream must already be positioned at the correct offset
      */
     KServiceType(QDataStream &_str, int offset);
 
     Q_DECLARE_PRIVATE(KServiceType)
 };
 
-//QDataStream& operator>>( QDataStream& _str, KServiceType& s );
-//QDataStream& operator<<( QDataStream& _str, KServiceType& s );
+// QDataStream& operator>>( QDataStream& _str, KServiceType& s );
+// QDataStream& operator<<( QDataStream& _str, KServiceType& s );
 
 #endif
